"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const fse = require("fs-extra");
const inquirer = require("inquirer");
const _ = require("lodash");
const ora = require("ora");
const path = require("path");
const webpack = require("webpack");
const devServer_1 = require("./devServer");
const inquirer_1 = require("./inquirer");
const utils_1 = require("./utils");
// import SentryUpload = require("sentry-uploader");
const Git_1 = require("./utils/Git");
const webpack_1 = require("./webpack");
function validateConfig(config) {
    return config;
}
exports.validateConfig = validateConfig;
function selectEnv(envList) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log();
        envList.forEach((env, index) => {
            console.log(`${index + 1}: ${env}`);
        });
        console.log();
        let answers = yield inquirer.prompt([
            {
                type: "input",
                name: "num",
                message: "Please input the env number"
            }
        ]);
        if (!_.isFinite(+answers.num)) {
            utils_1.printError("Invalid env num.");
            process.exit(1);
        }
        const env = envList[answers.num - 1];
        if (env) {
            return env;
        }
        else {
            utils_1.printError(`Env ${answers.num} not found.`);
            process.exit(1);
            return "";
        }
    });
}
class EZPack {
    constructor(options) {
        this.sentryRelease = "";
        this.options = options;
    }
    getExtendWebpackConfig() {
        return this.options.extendWebpackConfig === undefined
            ? ({ options }) => Promise.resolve(options)
            : this.options.extendWebpackConfig;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const { webpack: webpackConfig, name, devServer } = this.options;
            const config = yield this.getExtendWebpackConfig()({
                options: webpack_1.genDevConfig(webpackConfig, name, devServer.port)
            });
            devServer.name = name;
            devServer_1.default(config, devServer);
        });
    }
    build(analyse) {
        return __awaiter(this, void 0, void 0, function* () {
            const currentRepo = new Git_1.default();
            const currentBranch = yield currentRepo.getCurrentBranch();
            const isOnline = currentBranch === "master";
            const { webpack: webpackConfig, disableEntrySelect } = this.options;
            let { entry } = webpackConfig;
            const entryLen = Object.keys(entry).length;
            if (entryLen > 1) {
                entry = yield (disableEntrySelect === true
                    ? inquirer_1.getEntryByKey([inquirer_1.ALL_KEY], webpackConfig.entry, webpackConfig.rootPath)
                    : inquirer_1.readCustomEntry(webpackConfig.entry, webpackConfig.rootPath, true));
                if (!entry) {
                    utils_1.printWarn("No entry found.");
                    return;
                }
            }
            this.sentryRelease = (+Date.now()).toString();
            const config = yield this.getExtendWebpackConfig()({
                options: webpack_1.genProdConfig(Object.assign({}, webpackConfig, { entry }), this.options.name, isOnline, this.sentryRelease, analyse)
            });
            const spinner = ora("webpack building...");
            spinner.start();
            return new Promise((resolve, reject) => {
                webpack(config, (err, stats) => {
                    spinner.stop();
                    if (err) {
                        reject(err);
                    }
                    else if (stats.hasErrors()) {
                        reject(stats.toJson().errors.join("\n"));
                        utils_1.printError(`Build error：`);
                    }
                    else {
                        process.stdout.write(stats.toString({
                            colors: true,
                            modules: false,
                            children: false,
                            chunks: false,
                            chunkModules: false
                        }) + "\n\n");
                        console.log(chalk.cyan("  Build complete.\n"));
                        // if (entryLen > 1) {
                        // 	// only update auto-entry when having multiple entries
                        // 	updateAutoEntry();
                        // }
                        utils_1.alertSuccess("Build successfully.");
                        resolve();
                    }
                });
            });
        });
    }
    // private async uploadSentry(isOnline: boolean) {
    // 	const sentryConfig = this.options.sentry;
    // 	if (sentryConfig) {
    // 		const distPath = this.options.webpack.outputPath;
    // 		const mapFiles = await listFiles(distPath, ".map");
    // 		const { token, host } = sentryConfig;
    // 		const configKey = isOnline ? "online" : "uat";
    // 		const uploader = new SentryUpload(
    // 			this.sentryRelease,
    // 			"sentry",
    // 			sentryConfig.projectName,
    // 			token[configKey],
    // 			host[configKey]
    // 		);
    // 		await uploader.doUpload(path.join(distPath, "**/*.map"), distPath);
    // 		await removeFiles(mapFiles);
    // 	}
    // }
    publish() {
        return __awaiter(this, void 0, void 0, function* () {
            const { publish: publishOptions, webpack: webpackOptions, name: projectName } = this.options;
            const currentRepo = new Git_1.default();
            const currentBranch = yield currentRepo.getCurrentBranch();
            const isOnline = currentBranch === "master";
            let publishPath = "";
            let mapPath = "";
            let env = "";
            if (isOnline) {
                const answers = yield inquirer.prompt([
                    {
                        type: "confirm",
                        name: "isPublish",
                        default: false,
                        message: "Are you sure to publish online?"
                    }
                ]);
                if (!answers.isPublish) {
                    return;
                }
                env = "online";
                const { mapDir, onlinePath } = publishOptions;
                publishPath = onlinePath;
                if (mapDir) {
                    mapPath = path.join(publishPath, mapDir, "map.js");
                }
            }
            else {
                const { uatPath, uatEnv, uatMapName, mapDir } = publishOptions;
                if (uatEnv && uatEnv.length > 0) {
                    env = yield selectEnv(uatEnv);
                    publishPath = path.resolve(uatPath, env);
                }
                else {
                    publishPath = uatPath;
                    if (uatMapName) {
                        env = yield selectEnv(uatMapName);
                        mapPath = path.join(uatPath, mapDir, env);
                    }
                }
            }
            if (!(yield utils_1.isDirectory(publishPath))) {
                utils_1.printError(`publish fail! ${publishPath} not exists.`);
                return;
            }
            const publishRepo = new Git_1.default(publishPath);
            // make sure publishPath is a clean git repository
            try {
                const stats = yield publishRepo.status();
                if (stats.files.length !== 0) {
                    utils_1.printError(`${publishPath} is not clean, check it.`);
                    return;
                }
            }
            catch (e) {
                utils_1.printError(`${publishPath} is not a git repository.`);
                return;
            }
            publishRepo.pull("origin", "master");
            try {
                yield fse.emptyDir(webpackOptions.outputPath);
                yield this.build();
            }
            catch (e) {
                utils_1.printError(e);
                return;
            }
            // remove upload map to sentry
            // try {
            // 	await this.uploadSentry(isOnline);
            // } catch (e) {
            // 	console.dir(e);
            // 	printError("Upload sentry fail.");
            // }
            if (mapPath) {
                yield fse.move(path.join(webpackOptions.outputPath, "map.js"), mapPath, {
                    overwrite: true
                });
            }
            yield fse.copy(webpackOptions.outputPath, publishPath);
            try {
                yield publishRepo.add("./*");
                yield publishRepo.commit(`Auto publish ${projectName} ${currentBranch}.`);
                yield publishRepo.pull("origin", "master", {
                    "--rebase": "true"
                });
                yield publishRepo.push("master");
                utils_1.printSuccess(`Publish ${env} successfully.`);
            }
            catch (e) {
                utils_1.printWarn("Build successfully, but auto publish fail.");
                return;
            }
        });
    }
}
exports.EZPack = EZPack;
//# sourceMappingURL=EZPack.js.map