/**
 * ezpack的接口定义文件
 * 也是doc.html的生成源
 */
import { WebpackOptions } from "./webpack/interface";
import { DevServerOption } from "./devServer";
export interface PublishOptions {
    /**
     * 线上静态文件的仓库（比如 ezstatic）路径
     */
    onlinePath: string;
    /**
     * uat 静态文件的仓库（比如 localezstatic）路径
     */
    uatPath: string;
    /**
     * uat 对应的多套环境的名称，对应多个静态目录，比如 dgamin.65emall.net，dgadmin2.65emall.net
     */
    uatEnv: string[];
    /**
     * uat 对应的多个 map 文件的名称，比如 aspirin 项目的 env1-map.js，env2-map.js
     */
    uatMapName: string[];
    /**
     * 保存上面多个 map 文件的目录名称
     */
    mapDir: string;
}
export declare type ExtendWebpackConfig = ({ options: WebpackOptions }: {
    options: any;
}) => Promise<WebpackOptions>;
export interface EZPackOptions {
    /** 项目名称 */
    name: string;
    /**
     * webpack 相关配置
     */
    webpack: WebpackOptions;
    /**
     * 开发服务器配置
     */
    devServer: DevServerOption;
    /**
     * 发布相关配置
     */
    publish: PublishOptions;
    /**
     * 禁用start/ build 模式下的entry选择，默认使用All entry
     */
    disableEntrySelect?: boolean;
    /**
     * sentry 相关配置
     */
    sentry?: {
        /**
         * 项目在 sentry 上面的名称
         */
        projectName: string;
        /**
         * sentry 的域名，用于上传 map 文件
         */
        host: {
            online: string;
            uat: string;
        };
        token: {
            online: string;
            uat: string;
        };
    };
    extendWebpackConfig?: ExtendWebpackConfig;
}
