"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const CacheData_1 = require("../CacheData");
const utils_1 = require("../utils");
var EntryStatus;
(function (EntryStatus) {
    EntryStatus[EntryStatus["ADDED"] = 0] = "ADDED";
    EntryStatus[EntryStatus["BUILDING"] = 1] = "BUILDING";
    EntryStatus[EntryStatus["BUILT"] = 2] = "BUILT";
})(EntryStatus = exports.EntryStatus || (exports.EntryStatus = {}));
class Invalidator {
    constructor(devMiddleware) {
        this.devMiddleware = devMiddleware;
        this.building = false;
        this.rebuildAgain = false;
    }
    invalidate() {
        if (this.building) {
            this.rebuildAgain = true;
            return;
        }
        this.building = true;
        this.devMiddleware.invalidate();
    }
    startBuilding() {
        this.building = true;
    }
    doneBuilding() {
        this.building = false;
        if (this.rebuildAgain) {
            this.rebuildAgain = false;
            this.invalidate();
        }
    }
}
exports.Invalidator = Invalidator;
class RouteEntry {
    constructor(o) {
        this.addEntry = (name) => {
            CacheData_1.devEntryCache.addEntry(name);
            this.builtEntries[name] = EntryStatus.ADDED;
        };
        this.startBuild = () => {
            // 这里的顺序不能调换，必须先 invalidate 再 setBuilding
            this.invalidator.invalidate();
            this.setBuilding();
        };
        this.finishBuild = () => {
            Object.keys(this.builtEntries).forEach(page => {
                const status = this.builtEntries[page];
                if (status !== EntryStatus.BUILDING)
                    return;
                this.builtEntries[page] = EntryStatus.BUILT;
                this.doneCallbacks.emit(page);
                this.invalidator.doneBuilding();
                this.afterEntryBuild(page);
            });
        };
        this.checkRoute = (entryName, done) => {
            if (!this.allEntries[entryName]) {
                done();
                return;
            }
            const status = this.builtEntries[entryName];
            if (status) {
                if (status === EntryStatus.BUILT) {
                    done();
                }
                else if (status === EntryStatus.BUILDING) {
                    this.doneCallbacks.on(entryName, done);
                }
            }
            else {
                utils_1.printWarn(`> Building page: ${entryName}`);
                this.addEntry(entryName);
                this.doneCallbacks.on(entryName, done);
                this.startBuild();
            }
        };
        this.builtEntries = {};
        if (o.initEntry.length > 0) {
            o.initEntry.forEach(k => {
                this.builtEntries[k] = EntryStatus.BUILDING;
            });
        }
        this.invalidator = new Invalidator(o.devMiddleware);
        this.allEntries = o.allEntries;
        this.doneCallbacks = new events_1.EventEmitter();
        this.afterEntryBuild = o.afterEntryBuild;
        this.getEntriesForBuild = this.getEntriesForBuild.bind(this);
    }
    setBuilding() {
        this.invalidator.startBuilding();
        Object.keys(this.builtEntries).forEach(k => {
            this.builtEntries[k] = EntryStatus.BUILDING;
        });
    }
    getEntriesForBuild() {
        return Object.keys(this.builtEntries).map(name => ({
            name,
            entry: this.allEntries[name]
        }));
    }
}
exports.RouteEntry = RouteEntry;
//# sourceMappingURL=RouteEntry.js.map