"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const crypto = require("crypto");
const path = require("path");
const fse = require("fs-extra");
const os = require("os");
const notifier = require("node-notifier");
exports.printInfo = msg => {
    console.log();
    console.log(chalk.blue(msg));
};
exports.printError = msg => {
    console.log();
    console.log(chalk.red(msg));
};
exports.printSuccess = msg => {
    console.log();
    console.log(chalk.green(msg));
};
exports.printWarn = msg => {
    console.log();
    console.log(chalk.yellow(msg));
};
function isExist(pathName, directory) {
    return fse
        .stat(pathName)
        .then(stats => {
        if (stats) {
            if (directory) {
                return stats.isDirectory();
            }
            else {
                return stats.isFile();
            }
        }
        return false;
    })
        .catch(() => false);
}
function isDirectory(dirPathName) {
    return isExist(dirPathName, true);
}
exports.isDirectory = isDirectory;
function getLocalIP() {
    const ifaces = os.networkInterfaces();
    const ipList = [];
    Object.keys(ifaces).forEach(ifname => {
        let alias = 0;
        ifaces[ifname].forEach(iface => {
            if (iface.family !== "IPv4" || iface.internal !== false) {
                // skip over internal (i.e. 127.0.0.1) and non-ipv4 addresses
                return;
            }
            if (alias >= 1) {
                // this single interface has multiple ipv4 addresses
                console.log(ifname + ":" + alias, iface.address);
            }
            else {
                // this interface has only one ipv4 adress
                ipList.push(iface.address);
            }
            ++alias;
        });
    });
    let ip = "";
    if (ipList.length > 0) {
        ip = ipList.find(item => item.includes("192"));
    }
    if (ip) {
        return ip;
    }
    else {
        exports.printWarn("Can not get your local ip, using 127.0.0.1");
        return "127.0.0.1";
    }
}
exports.localIP = getLocalIP();
function alertSuccess(message) {
    notifier.notify({
        title: "ezpack",
        message
    });
}
exports.alertSuccess = alertSuccess;
/**
 * 递归遍历目录下所有文件
 * @param 要递归遍历的目录
 * @param 文件后缀类型
 */
function listFiles(listPath, ext) {
    return __awaiter(this, void 0, void 0, function* () {
        if (yield isDirectory(listPath)) {
            let files = yield fse.readdir(listPath);
            let allFiles = [];
            for (let i = 0; i < files.length; i++) {
                const subFiles = yield listFiles(path.join(listPath, files[i]), ext);
                allFiles = allFiles.concat(subFiles);
            }
            return Promise.resolve(allFiles);
        }
        else {
            if (!ext || path.extname(listPath) === ext) {
                return Promise.resolve([listPath]);
            }
            else {
                return Promise.resolve([]);
            }
        }
    });
}
exports.listFiles = listFiles;
function removeFiles(filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        return Promise.all(filePath.map(f => fse.unlink(f)));
    });
}
exports.removeFiles = removeFiles;
function hash(content, len) {
    const hashStr = crypto
        .createHash("md5")
        .update(content)
        .digest("hex");
    if (len) {
        return hashStr.slice(0, len);
    }
    else {
        return hashStr;
    }
}
exports.hash = hash;
//# sourceMappingURL=index.js.map