"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const webpack = require("webpack");
const config_1 = require("../../config");
const genUglifyJsPlugin_1 = require("./genUglifyJsPlugin");
function genDllConfig(options, isProd) {
    const mode = isProd ? "production" : "development";
    const version = typeof options.version === "string" ? `.${options.version}` : "";
    const filename = isProd ? `js/[name]${version}.[chunkhash:${config_1.HASH_LEN}].dll.js` : "[name].dll.js";
    const library = "[name]_lib";
    return {
        entry: options.dllEntry,
        devtool: isProd && options.sourceMap ? "source-map" : false,
        output: {
            filename,
            path: options.outputPath,
            publicPath: isProd ? options.cdnPath : "",
            library
        },
        externals: options.externals || {},
        module: {
            rules: [
                {
                    test: /\.css/,
                    use: ["style-loader", "css-loader"]
                }
            ]
        },
        mode,
        optimization: isProd
            ? {
                minimizer: [genUglifyJsPlugin_1.genUglifyJsPlugin(options.sourceMap, config_1.HOME_DIR)]
            }
            : {},
        plugins: [
            new webpack.DllPlugin({
                path: "[name]-manifest.json",
                name: library,
                context: options.outputPath
            }),
            // remove moment locale, except en and zh
            new webpack.ContextReplacementPlugin(/moment[\/\\]locale$/, /en-gb|zh-cn/)
        ].concat(isProd
            ? [new webpack.HashedModuleIdsPlugin(), new webpack.optimize.ModuleConcatenationPlugin()]
            : [])
    };
}
exports.genDllConfig = genDllConfig;
exports.default = genDllConfig;
//# sourceMappingURL=genDllConfig.js.map