"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// import * as path from "path";
const DllLinkPlugin = require("dll-link-webpack-plugin");
const ForkTsCheckerWebpackPlugin = require("fork-ts-checker-webpack-plugin");
const webpack = require("webpack");
const HtmlPlugin_1 = require("../Plugin/HtmlPlugin");
const MapPlugin_1 = require("../Plugin/MapPlugin");
const genDllConfig_1 = require("./genDllConfig");
exports.genPlugins = (options, isProd, devPort) => {
    // const root = process.cwd();
    // const tsconfigPath = path.resolve(root, "./tsconfig.json");
    // const tslintPath = path.resolve(root, "./tslint.json");
    // console.log("tsconfig:", tsconfigPath);
    // console.log("tslint: ", tslintPath);
    const plugins = [
        // https://github.com/clinyong/dll-link-webpack-plugin/blob/master/README-zh_CN.md
        new DllLinkPlugin({
            config: genDllConfig_1.default(options, isProd),
            appendVersion: isProd,
            assetsMode: true,
            htmlMode: true // true | false 如果你用了 html-webpack-plugin，生成出来的 DLL 文件会被自动引入 html 文件中。（默认值为 false）
        }),
        // remove moment locale, except en and zh
        // see https://github.com/moment/moment/issues/2416#issuecomment-344840418
        new webpack.ContextReplacementPlugin(/moment[\/\\]locale$/, /en-gb|zh-cn/),
        new ForkTsCheckerWebpackPlugin({
            // async: false,
            // tsconfig: tsconfigPath,
            // tslint: tslintPath,
            memoryLimit: 3072,
            workers: ForkTsCheckerWebpackPlugin.ONE_CPU
        })
    ];
    if (options.html) {
        const { htmlOption, entry, dllEntry, cdnPath } = options;
        const htmlConfig = {
            entry,
            dllEntry,
            isProd,
            cdnPath
        };
        if (htmlOption) {
            const { title, template, entryTemplates, preload, disableMinify, customTplData } = htmlOption;
            if (title) {
                htmlConfig.title = title;
            }
            if (template) {
                htmlConfig.template = template;
            }
            if (entryTemplates) {
                htmlConfig.entryTemplates = entryTemplates;
            }
            if (preload) {
                htmlConfig.preload = preload;
            }
            if (disableMinify) {
                htmlConfig.disableMinify = disableMinify;
            }
            htmlConfig.customTplData = customTplData;
        }
        plugins.push(new HtmlPlugin_1.default(htmlConfig));
    }
    else {
        plugins.push(new MapPlugin_1.default({
            name: "map",
            dllEntry: options.dllEntry,
            isProd,
            devPort
        }));
    }
    return plugins.concat(Array.isArray(options.plugins) ? options.plugins : []);
};
//# sourceMappingURL=index.js.map