"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
function isParentDir(parentPath, childPath) {
    return !path.relative(parentPath, childPath).startsWith(`..${path.sep}`);
}
function genStyleLoaders(isProd, modulePath, globalPath) {
    const moduleName = isProd ? "[hash:base64:5]" : "[folder]--[local]--[hash:base64:5]";
    const cssModuleLoader = {
        loader: "css-loader",
        options: {
            modules: true,
            localIdentName: moduleName,
            importLoaders: 1
        }
    };
    const postCSSLoader = {
        loader: "postcss-loader",
        options: {
            plugins() {
                return [require("autoprefixer")];
            }
        }
    };
    const cssModulesUse = ["style-loader", cssModuleLoader, postCSSLoader];
    const cssGlobalUse = ["style-loader", "css-loader", postCSSLoader];
    const styleLoaders = [];
    const nodeModulesPath = path.resolve("./node_modules");
    if (modulePath) {
        const { include, exclude } = modulePath;
        styleLoaders.push({
            test: /\.css$/,
            include,
            exclude,
            use: cssModulesUse
        }, {
            test: /\.scss$/,
            include,
            exclude,
            use: cssModulesUse.concat("fast-sass-loader")
        });
    }
    if (globalPath) {
        const { include, exclude } = globalPath;
        styleLoaders.push({
            test: /\.css$/,
            include,
            exclude,
            use: cssGlobalUse
        }, {
            test: /\.scss$/,
            include,
            exclude,
            use: cssGlobalUse.concat("fast-sass-loader")
        });
    }
    if (styleLoaders.length === 0) {
        // 如果没有配置 css 路径，默认对非 node_modules 进行 css modules 处理
        styleLoaders.push({
            test: /\.css$/,
            exclude: nodeModulesPath,
            use: cssModulesUse
        }, {
            test: /\.scss$/,
            exclude: nodeModulesPath,
            use: cssModulesUse.concat("fast-sass-loader")
        });
    }
    // 默认会处理 node_modules 里面的 css 文件
    let needProcessNodeModulesGlobalCSS = true;
    if (globalPath !== undefined && globalPath.include !== undefined) {
        const paths = !Array.isArray(globalPath.include) ? [globalPath.include] : globalPath.include;
        needProcessNodeModulesGlobalCSS =
            paths.find(path => path === nodeModulesPath || isParentDir(path, nodeModulesPath)) ===
                undefined;
    }
    if (needProcessNodeModulesGlobalCSS) {
        styleLoaders.push({
            test: /\.css$/,
            include: nodeModulesPath,
            use: cssGlobalUse
        });
    }
    return styleLoaders;
}
exports.genStyleLoaders = genStyleLoaders;
//# sourceMappingURL=StyleLoaders.js.map