"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../../config");
const StyleLoaders_1 = require("./StyleLoaders");
const prodNameTPL = `[name].[hash:${config_1.HASH_LEN}].[ext]`;
function genRules(options, isProd) {
    let jsLoaders = [
        {
            loader: "babel-loader",
            options: {
                cacheDirectory: true,
                // exclude: /node_modules/,
                // include: SRC_PATH,
                presets: [
                    [
                        "env",
                        {
                            modules: false
                        }
                    ],
                    "react",
                    "stage-1"
                ],
                plugins: [
                    ...(options.babelPlugins ? options.babelPlugins : []),
                    "lodash",
                    [
                        "import",
                        [
                            {
                                libraryName: "antd",
                                libraryDirectory: "es"
                            }
                        ]
                    ],
                    "transform-decorators-legacy"
                ],
                comments: false
            }
        }
    ];
    if (!isProd) {
        jsLoaders = ["react-hot-loader/webpack", "cache-loader", ...jsLoaders];
    }
    const tsLoaders = [
        ...jsLoaders,
        {
            loader: "ts-loader",
            // https://webpack.js.org/guides/build-performance/#typescript-loader
            // 为了增加构建速度
            options: {
                transpileOnly: true,
                happyPackMode: true
            }
        }
    ];
    const { cssModulePath, cssGlobalPath } = options;
    return [
        {
            test: /\.tsx?$/,
            include: options.tsInclude || options.rootPath,
            use: tsLoaders
        },
        {
            test: /\.jsx?$/,
            include: options.jsInclude || options.rootPath,
            use: jsLoaders
        },
        {
            test: /.(woff(2)?|eot|ttf|svg)(\?[a-z0-9=.]+)?$/,
            use: [
                {
                    loader: "url-loader",
                    options: {
                        limit: 8192,
                        name: isProd ? `fonts/${prodNameTPL}` : "fonts/[name].[ext]"
                    }
                }
            ]
        },
        {
            test: /.(png|jpg)(\?[a-z0-9=.]+)?$/,
            use: [
                {
                    loader: "url-loader",
                    options: {
                        limit: 8192,
                        name: isProd ? `images/${prodNameTPL}` : "images/[name].[ext]"
                    }
                }
            ]
        },
        ...StyleLoaders_1.genStyleLoaders(isProd, cssModulePath, cssGlobalPath)
    ];
}
exports.genRules = genRules;
//# sourceMappingURL=index.js.map