"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("../config");
const Plugins_1 = require("./Plugins");
const Rules_1 = require("./Rules");
function genConfig(options, name, isProd, devPort) {
    const polyfillNames = ["babel-polyfill", "core-js"];
    const polyfillName = Object.keys(options.dllEntry).reduce(function (pValue, cValue) {
        if (pValue === "") {
            const entry = options.dllEntry[cValue];
            if (typeof entry === "string" && polyfillNames.includes(entry)) {
                return entry;
            }
            else if (Array.isArray(entry)) {
                const result = entry.find(item => polyfillNames.includes(item));
                if (typeof result !== "undefined") {
                    return result;
                }
            }
        }
        return pValue;
    }, "");
    let entryPrefix = [];
    if (polyfillName !== "") {
        entryPrefix.push(polyfillName);
    }
    if (!isProd) {
        entryPrefix.push("react-hot-loader/patch", `webpack-hot-middleware/client?path=http://${config_1.localIP}:${devPort}/__webpack_hmr&overlay=false`);
    }
    if (entryPrefix.length > 0) {
        Object.keys(options.entry).forEach(function (name) {
            options.entry[name] = entryPrefix.concat(options.entry[name]);
        });
    }
    return {
        entry: options.entry,
        resolve: {
            extensions: [".webpack.js", ".js", ".jsx", ".tsx", ".ts"],
            modules: [options.rootPath, "node_modules", config_1.MODULE_PATH]
        },
        resolveLoader: {
            modules: ["node_modules", config_1.MODULE_PATH]
        },
        mode: isProd ? "production" : "development",
        module: {
            noParse(content) {
                return /jquery|lodash/.test(content);
            },
            rules: Rules_1.genRules(options, isProd)
        },
        plugins: Plugins_1.genPlugins(options, isProd, devPort),
        externals: options.externals
    };
}
exports.genConfig = genConfig;
exports.default = genConfig;
//# sourceMappingURL=base.config.js.map