"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const CopyWebpackPlugin = require("copy-webpack-plugin");
const webpack = require("webpack");
const webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
const merge = require("webpack-merge");
const DefineGlobalPlugin_1 = require("./Plugin/DefineGlobalPlugin");
const config_1 = require("../config");
const genUglifyJsPlugin_1 = require("./Plugins/genUglifyJsPlugin");
const base_config_1 = require("./base.config");
function genProdConfig(options, name, isOnline, sentryRelease, analyse) {
    let { cdnPath, splitChunks } = options;
    if (typeof cdnPath === "object") {
        if (isOnline) {
            cdnPath = cdnPath.online;
        }
        else {
            cdnPath = cdnPath.uat;
        }
    }
    options.cdnPath = cdnPath;
    const baseConfig = base_config_1.genConfig(options, name, true);
    const sourceMap = !!options.sourceMap;
    const optimization = {
        splitChunks,
        minimizer: [genUglifyJsPlugin_1.default(sourceMap, config_1.HOME_DIR)]
    };
    const plugins = [
        new webpack.DefinePlugin({
            "process.env.NODE_ENV": JSON.stringify("production")
        }),
        // remove moment locale, except en and zh
        // see https://github.com/moment/moment/issues/2416#issuecomment-344840418
        new webpack.ContextReplacementPlugin(/moment[\/\\]locale$/, /en-gb|zh-cn/),
        new webpack.HashedModuleIdsPlugin(),
        new DefineGlobalPlugin_1.DefineGlobalPlugin({
            SENTRY_RELEASE: sentryRelease
        })
    ];
    if (splitChunks === undefined) {
        optimization.splitChunks = {
            chunks: "initial",
            minChunks: 2,
            cacheGroups: {
                commons: {
                    name: "common",
                    chunks: "initial",
                    minChunks: 4
                }
            }
        };
    }
    optimization.runtimeChunk = true;
    if (analyse) {
        plugins.push(new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
            analyzerMode: "static"
        }));
    }
    else {
        // exclude ModuleConcatenationPlugin in analyse mode
        // see https://github.com/webpack-contrib/webpack-bundle-analyzer#i-cant-see-all-the-dependencies-in-a-chunk
        plugins.push(new webpack.optimize.ModuleConcatenationPlugin());
    }
    if (Array.isArray(options.copyPath) && options.cdnPath.length > 0) {
        plugins.push(new CopyWebpackPlugin(options.copyPath));
    }
    const version = typeof options.version === "string" ? `.${options.version}` : "";
    const nameTPL = `js/[name]${version}.[chunkhash:${config_1.HASH_LEN}]`;
    return merge(baseConfig, {
        output: {
            path: options.outputPath,
            publicPath: cdnPath,
            filename: `${nameTPL}.js`,
            chunkFilename: `${nameTPL}.chunk.js`
        },
        devtool: sourceMap ? "source-map" : false,
        plugins,
        optimization
    });
}
exports.genProdConfig = genProdConfig;
//# sourceMappingURL=prod.config.js.map