import * as webpack from "webpack";
import { addWebpackEntry } from "../utils/webpack";

export interface EntryForBuild {
	name: string;
	entry: string;
}

export interface RouteEntryPluginOptions {
	getEntriesForBuild: () => EntryForBuild[];
	done: () => void;
}

export class RouteEntryPlugin {
	options: RouteEntryPluginOptions;

	constructor(options: RouteEntryPluginOptions) {
		this.options = options;
	}

	apply(compiler: webpack.Compiler) {
		const name = "RouteEntryPlugin";

		compiler.hooks.make.tapAsync(name, (compilation, done) => {
			const allEntries = this.options
				.getEntriesForBuild()
				.map(({ name, entry }) =>
					addWebpackEntry(compilation, (compiler as any).context, name, entry)
				);

			Promise.all(allEntries)
				.then(() => done())
				.catch(done);
		});

		compiler.hooks.done.tap(name, this.options.done);
	}
}
