import * as webpack from "webpack";
import { getDashboard } from ".";

const PLUGIN_NAME = "DashboardPlugin";
export class DashboardPlugin {
	apply(compiler: webpack.Compiler) {
		const d = getDashboard();
		const send = d.setData.bind(d);
		const h = compiler.hooks;

		new webpack.ProgressPlugin((percent, msg) => {
			send([
				{
					type: "status",
					value: "Compiling"
				},
				{
					type: "progress",
					value: percent
				},
				{
					type: "operations",
					value: msg
				}
			]);
		}).apply(compiler);

		h.compile.tap(PLUGIN_NAME, () => {
			send([
				{
					type: "status",
					value: "Compiling"
				}
			]);
		});
		h.invalid.tap(PLUGIN_NAME, () => {
			send([
				{
					type: "status",
					value: "Invalidated"
				},
				{
					type: "progress",
					value: 0
				},
				{
					type: "operations",
					value: "idle"
				},
				{
					type: "clear"
				}
			]);
		});
		h.failed.tap(PLUGIN_NAME, () => {
			send([
				{
					type: "status",
					value: "Failed"
				},
				{
					type: "operations",
					value: "idle"
				}
			]);
		});
		h.done.tap(PLUGIN_NAME, stats => {
			send([
				{
					type: "status",
					value: "Success"
				},
				{
					type: "progress",
					value: 0
				},
				{
					type: "operations",
					value: "idle"
				},
				{
					type: "stats",
					value: stats
				}
			]);
		});
	}
}
