import * as ora from "ora";
import * as fse from "fs-extra";
import * as inquirer from "inquirer";
import * as SearchCheckbox from "inquirer-search-checkbox";
import Git from "../utils/Git";
import { getPackages } from "../utils/parse";
import { printInfo } from "../utils";
import { HOME_DIR } from "../config";
import { lastBuiltEntryCache } from "../CacheData";
import { ALL_KEY } from "./index";

inquirer.registerPrompt("SearchCheckbox", SearchCheckbox);

const CACHE_FILE = `./${HOME_DIR}/auto-entry`;
const git = new Git();

const AUTO_KEY = "auto";
const LAST_KEY = "<LAST>";

function getAutoEntry(): string {
	fse.ensureFileSync(CACHE_FILE);
	return fse.readFileSync(CACHE_FILE, "utf8").replace("\n", "");
}

async function autoSelectEntry(
	entry: {
		[index: string]: string[];
	},
	rootPath: string
) {
	const allPackageMap: {
		[index: string]: object;
	} = {};
	const entryKeyList = Object.keys(entry);
	const spinner = ora("Finding entries...");
	spinner.start();
	return Promise.all(
		Object.keys(entry).map(k => {
			const item = entry[k];
			let entryFile: string | string[] = item;
			if (Array.isArray(item)) {
				entryFile = item.reverse()[0];
			}
			allPackageMap[k] = {};
			return getPackages(allPackageMap[k], entryFile as string, rootPath);
		})
	).then(async packages => {
		const saveCommit = getAutoEntry();
		const latestCommit = await git.getLatestCommit();
		spinner.stop();
		if (saveCommit && saveCommit !== latestCommit) {
			const files = await git.getFilesByCommitRange(saveCommit, latestCommit);
			const changeKeyList = entryKeyList.filter(k => files.some(file => allPackageMap[k][file]));

			if (changeKeyList.length > 0) {
				console.log("Below entries will be built: ");
				changeKeyList.forEach(k => {
					console.log("    - " + k);
				});
				console.log();
				return changeKeyList.reduce((obj, k) => {
					obj[k] = entry[k];
					return obj;
				}, {});
			}

			return null;
		}

		return null;
	});
}

export async function getEntryByKey(key: string[], entry: any, rootPath: string) {
	if (key.includes(ALL_KEY)) {
		return Promise.resolve(entry);
	} else if (key.includes(AUTO_KEY)) {
		return await autoSelectEntry(entry, rootPath);
	} else if (key.includes(LAST_KEY)) {
		const lastEntry = lastBuiltEntryCache.getEntry();

		if (lastEntry.length === 0) {
			return Promise.resolve(null);
		} else {
			key = lastBuiltEntryCache.getEntry();
			printInfo(`Building entry: ${key.join(", ")}`);
		}
	}

	lastBuiltEntryCache.refreshEntry(key);
	const outputEntry = key.reduce(function(pValue, cValue) {
		if (typeof entry[cValue] !== "undefined") {
			pValue[cValue] = entry[cValue];
		}
		return pValue;
	}, {});
	return Promise.resolve(outputEntry);
}
