import * as webpack from "webpack";

import { HASH_LEN, HOME_DIR } from "../../config";
import { WebpackOptions } from "../interface";
import { genUglifyJsPlugin } from "./genUglifyJsPlugin";

export function genDllConfig(options: WebpackOptions, isProd: boolean): any {
	const mode = isProd ? "production" : "development";
	const version = typeof options.version === "string" ? `.${options.version}` : "";
	const filename = isProd ? `js/[name]${version}.[chunkhash:${HASH_LEN}].dll.js` : "[name].dll.js";

	const library = "[name]_lib";
	return {
		entry: options.dllEntry,
		devtool: isProd && options.sourceMap ? "source-map" : false,
		output: {
			filename,
			path: options.outputPath,
			publicPath: isProd ? options.cdnPath : "",
			library
		},
		externals: options.externals || {},
		module: {
			rules: [
				{
					test: /\.css/,
					use: ["style-loader", "css-loader"]
				}
			]
		},
		mode,
		optimization: isProd
			? {
					minimizer: [genUglifyJsPlugin(options.sourceMap, HOME_DIR)]
			  }
			: {},
		plugins: [
			new webpack.DllPlugin({
				path: "[name]-manifest.json",
				name: library,
				context: options.outputPath
			}),
			// remove moment locale, except en and zh
			new webpack.ContextReplacementPlugin(/moment[\/\\]locale$/, /en-gb|zh-cn/)
		].concat(
			isProd
				? [new webpack.HashedModuleIdsPlugin(), new webpack.optimize.ModuleConcatenationPlugin()]
				: []
		)
	};
}

export default genDllConfig;
