// import * as path from "path";
import DllLinkPlugin = require("dll-link-webpack-plugin");
import * as ForkTsCheckerWebpackPlugin from "fork-ts-checker-webpack-plugin";
import * as webpack from "webpack";
import HtmlPlugin from "../Plugin/HtmlPlugin";
import MapPlugin from "../Plugin/MapPlugin";
import { WebpackOptions } from "../interface";
import genDllConfig from "./genDllConfig";

export const genPlugins = (options: WebpackOptions, isProd: boolean, devPort?: number) => {
	// const root = process.cwd();
	// const tsconfigPath = path.resolve(root, "./tsconfig.json");
	// const tslintPath = path.resolve(root, "./tslint.json");
	// console.log("tsconfig:", tsconfigPath);
	// console.log("tslint: ", tslintPath);
	const plugins: any[] = [
		// https://github.com/clinyong/dll-link-webpack-plugin/blob/master/README-zh_CN.md
		new DllLinkPlugin({
			config: genDllConfig(options, isProd),
			appendVersion: isProd, // true | false 给每个 webpack 生成出的 entry 文件打上一个版本号。（默认值为 false）
			assetsMode: true, // true | false 把 DLL 文件输出为 webpack 的 assets 文件。（默认值为 false）
			htmlMode: true // true | false 如果你用了 html-webpack-plugin，生成出来的 DLL 文件会被自动引入 html 文件中。（默认值为 false）
		}),
		// remove moment locale, except en and zh
		// see https://github.com/moment/moment/issues/2416#issuecomment-344840418
		new webpack.ContextReplacementPlugin(/moment[\/\\]locale$/, /en-gb|zh-cn/),
		new ForkTsCheckerWebpackPlugin({
			// async: false,
			// tsconfig: tsconfigPath,
			// tslint: tslintPath,
			memoryLimit: 3072,
			workers: ForkTsCheckerWebpackPlugin.ONE_CPU
		})
	];

	if (options.html) {
		const { htmlOption, entry, dllEntry, cdnPath } = options;
		const htmlConfig: any = {
			entry,
			dllEntry,
			isProd,
			cdnPath
		};
		if (htmlOption) {
			const { title, template, entryTemplates, preload, disableMinify, customTplData } = htmlOption;

			if (title) {
				htmlConfig.title = title;
			}
			if (template) {
				htmlConfig.template = template;
			}
			if (entryTemplates) {
				htmlConfig.entryTemplates = entryTemplates;
			}
			if (preload) {
				htmlConfig.preload = preload;
			}
			if (disableMinify) {
				htmlConfig.disableMinify = disableMinify;
			}

			htmlConfig.customTplData = customTplData;
		}
		plugins.push(new HtmlPlugin(htmlConfig));
	} else {
		plugins.push(
			new MapPlugin({
				name: "map",
				dllEntry: options.dllEntry,
				isProd,
				devPort
			})
		);
	}
	return plugins.concat(Array.isArray(options.plugins) ? options.plugins : []);
};
