import * as webpack from "webpack";

import { localIP, MODULE_PATH } from "../config";

import { WebpackOptions } from "./interface";

import { genPlugins } from "./Plugins";
import { genRules } from "./Rules";

export function genConfig(
	options: WebpackOptions,
	name: string,
	isProd: boolean,
	devPort?: number
): webpack.Configuration & {
	entry: webpack.Entry;
} {
	const polyfillNames = ["babel-polyfill", "core-js"];
	const polyfillName = Object.keys(options.dllEntry).reduce(function(pValue, cValue) {
		if (pValue === "") {
			const entry = options.dllEntry[cValue];
			if (typeof entry === "string" && polyfillNames.includes(entry)) {
				return entry;
			} else if (Array.isArray(entry)) {
				const result = entry.find(item => polyfillNames.includes(item));
				if (typeof result !== "undefined") {
					return result;
				}
			}
		}
		return pValue;
	}, "");
	let entryPrefix: string[] = [];
	if (polyfillName !== "") {
		entryPrefix.push(polyfillName);
	}
	if (!isProd) {
		entryPrefix.push(
			"react-hot-loader/patch",
			`webpack-hot-middleware/client?path=http://${localIP}:${devPort}/__webpack_hmr&overlay=false`
		);
	}
	if (entryPrefix.length > 0) {
		Object.keys(options.entry).forEach(function(name) {
			options.entry[name] = entryPrefix.concat(options.entry[name]);
		});
	}

	return {
		entry: options.entry,
		resolve: {
			extensions: [".webpack.js", ".js", ".jsx", ".tsx", ".ts"],
			modules: [options.rootPath, "node_modules", MODULE_PATH]
		},
		resolveLoader: {
			modules: ["node_modules", MODULE_PATH]
		},
		mode: isProd ? "production" : "development",
		module: {
			noParse(content) {
				return /jquery|lodash/.test(content);
			},
			rules: genRules(options, isProd)
		},
		plugins: genPlugins(options, isProd, devPort),
		externals: options.externals
	};
}

export default genConfig;
