import * as webpack from "webpack";
import * as merge from "webpack-merge";

import { DashboardPlugin } from "../devServer/dashboard/DashboardPlugin";

import { localIP } from "../config";
import { genConfig } from "./base.config";
import { WebpackOptions } from "./interface";

export function genDevConfig(options: WebpackOptions, name: string, devPort: number) {
	let { cdnPath } = options;
	if (typeof cdnPath === "object") {
		cdnPath = cdnPath.uat;
	}
	options.cdnPath = cdnPath;
	const baseConfig = genConfig(options, name, false, devPort);
	return merge(baseConfig, {
		output: {
			publicPath: `http://${localIP}:${devPort}/`,
			filename: "[name].js",
			chunkFilename: "[name].chunk.js",
			path: options.outputPath
		},
		devtool: "cheap-module-eval-source-map",
		plugins: [
			new webpack.DefinePlugin({
				"process.env.NODE_ENV": JSON.stringify("development")
			}),
			new webpack.NamedModulesPlugin(),
			new webpack.HotModuleReplacementPlugin(),
			new DashboardPlugin()
		]
	});
}
